/*global setLocalVar*/
var engine = function (eventAggregator, ScreenService, MessageService, General, LibLoginController, ZhuAclController, LibUtilities, restEngine, templateManager, ZHPromise) {

    const updateToken = (httpRequestResponse) => {
        if(httpRequestResponse.dataset){
            const token = httpRequestResponse.dataset.JWT[0];
            setLocalVar('token', token);
        }
    };
    let TColorToRGB = function (TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return '(' + r + ',' + g + ',' + b + ')';
    };

    var self = this;

    var aggregatorEvents = {
        BEFORE_PROJECT_STARTUP: "beforeProjectStartup",
        ON_CONTAINER_OPEN: "onContainerOpen",
        REQUEST_ON_START: "EventOnRequestStart"
    };
    var messages = {
        logout: "MSG_LOGGOUT"
    };
    var messageTypes = {
        ERROR: "E",
        ALERT: "W"
    };

    eventAggregator.subscribe(aggregatorEvents.BEFORE_PROJECT_STARTUP, validateSession);

    eventAggregator.subscribe(aggregatorEvents.ON_CONTAINER_OPEN, onContainerOpen);
    
    eventAggregator.subscribe("EventOnRequestMetaDataStart", changeUrlWeb);
    
    function changeUrlWeb() {
        const RELATIVE_URL = Util.getNestedProperty(arguments, ["0", "requestConfig", "url"]);
        if(typeof RELATIVE_URL == 'string' && RELATIVE_URL.toLowerCase().indexOf('login?params')>-1){
            const PREFIX = './assets/json/containers/login.json?params=';
            const PATTERN = /(?<PREFIX>params=)(?<TOKEN>[^\.]+)(?<SUFFIX>\.\w+)?/gi;
            let args = PATTERN.exec(RELATIVE_URL);
            let TOKEN = '';
            if(args && args.groups && args.groups.TOKEN) {
                TOKEN = args.groups.TOKEN;
            }
            arguments[0].requestConfig.url = PREFIX+TOKEN;
        }
    }

    function addTokenToRequest(requestData){
        const jwt = getLocalVar('token')
        if(jwt){
            const {requestConfig} = requestData;
            requestConfig.headers['token'] = jwt;
        }
    }


    eventAggregator.subscribe("EventOnRequestStart", addTokenToRequest);
    function onContainerOpen(event) {
        if (event && event.container && event.container.label) {
            templateManager.updateTitle(event.container.label);
            General.callRestEngine("/authenticator", "Row", {}).then(
                function (response) {
                    if(response.length > 0) {
                        if (!response.dataset.user[0]) {
                            errorMsg = response.error;
                            ScreenService.showMessage(ScreenService.i18n('Usúario inativo'), 'E');
                            forceLogOut();
                        }
                    }
                }
            ).catch();
        }
    }

    let __requestValidateSession = () => {
        return General.callRestEngine("/validateSession", "Row", {}).then(
            function (response) {
                self.openWindow("product#home");
                updateToken(response);
            },
            function () {
                setLocalVar('token', null);
                var url_string = window.location.href;
                var url = new URL(url_string);
                var cript = url.hash.split("params=")[1];
                if(cript == undefined) {
                    self.openWindow("product#login");    
                }
            }
        );
    };

    function validateSession() {
        if (getLocalVar('validQRCode') && getLocalVar('backendURL')) {
            return new Promise((resolve) => {
                let newDNS = getLocalVar('backendURL');
                templateManager.updateURL(newDNS);
                __requestValidateSession().then((requestResult) => {
                    resolve(requestResult);
                }).catch(() => {
                    let newDNSHttps = newDNS.replace("http:", "https:");
                    templateManager.updateURL(newDNSHttps);
                    __requestValidateSession().then((requestResult) => {
                        resolve(requestResult);
                    })
                })
            }).then(() => {
                let qrCodeData = getLocalVar('qrCodeData');
                if (qrCodeData) {
                    General.callRestEngine("/updateQrcodeConfig", "Row", qrCodeData).then((response) => {
                        if (response && response.dataset && response.dataset.qrcode) {
                            let qrcode = response.dataset.qrcode;
                            setLocalVar('IDPERANON', qrcode.IDPERANON);
                            setLocalVar('IDEXIBENUTR', qrcode.IDEXIBENUTR);
                            setLocalVar('IDTAMCARDIMG', qrcode.IDTAMCARDIMG);
                            setLocalVar('DSCORPRIMARIA', qrcode.DSCORPRIMARIA || '#E63635');
                            if (qrcode.DSCORPRIMARIA) {
                                document.documentElement.style.setProperty('--secundaryColor', 'rgb' + TColorToRGB(qrcode.DSCORPRIMARIA));
                            } else {
                                document.documentElement.style.setProperty('--secundaryColor', getLocalVar('DSCORPRIMARIA'));
                            }

                            setLocalVar('NRLOCALPESQ', qrcode.NRLOCALPESQ);
                            setLocalVar('qrCodeURL', qrcode.URLLOGO);
                            setLocalVar('DSNOMEEXIBICAO', qrcode.DSNOMEEXIBICAO);
                            setLocalVar('customerID', qrcode.CUSTOMERID);

                            let userData = getLocalVar('userData');
                            if ((userData && userData.isAnon) && qrcode.IDPERANON === 'N') {
                                forceLogOut();
                                MessageService.showMessageByCode("MSG_UNITY_DENIED_ANON_ACCESS");
                            }
                            if (window.shouldUpdateFCMToken) {
                                let fcmToken = localStorage.getItem('fcmToken');
                                sendFCMTokenSaveRequest(fcmToken, false).catch(() => sendFCMTokenSaveRequest(fcmToken, true));
                            }
                        }
                    });
                }
            });
        } else {
            var url_string = window.location.href;
            var url = new URL(url_string);
            var cript = url.hash.split("params=")[1];
            if(cript == undefined) {
                self.openWindow("product#login");   
                return ZHPromise.defer().reject();
            }
        }
    }

    function forceLogOut() {
        setLocalVar('token', null);
        self.openWindow("product#login");
        setLocalVar('userData', []);
        removeDevice();
    }

    this.forceLogOut = forceLogOut;

    this.logout = function () {
        MessageService.confirmMessageByCode(messages.logout, null, { 
            type: messageTypes.ALERT,
            success: function () {
                self.openWindow("product#login");
                setLocalVar('token', null);
                setLocalVar('userData', []);
                removeDevice();
            }.bind(this)
        });
    };


    function callbackError(err) {
        if (err.data && err.data.error) {
            ScreenService.showMessage(err.data.error, messageTypes.ERROR);
        } else {
            ScreenService.showMessage(JSON.stringify(err), messageTypes.ERROR);
        }
    }

    var requestModulesMetadata = function () {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };

    this.openWindow = function (windowName, newWindow) {
        requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            ScreenService.openWindow(windowName);
            // if (ScreenService.openLink) {
            //     ScreenService.openLink(url, newWindow);
            // } else {
            //     ScreenService.openWindow(windowName);
            // }
        });
    };

    let sendFCMTokenSaveRequest = this.sendFCMTokenSaveRequest = (token, __is_new = false) => {
        if ('device' in window) { // if isn't running in device do nothing
            let userData = getLocalVar('userData');
            return restEngine.post('/crud/device/save', {
                "requestType": "Row",
                "row": {
                    UUID: device.uuid,
                    PLATFORM: device.platform,
                    TOKEN: token,
                    // NRUSUARIOAPLICATIVO: userData.NRUSUARIOAPP,
                    CDOPERADOR: null,
                    NRORG: localStorage.getItem('NRORG').replace(/"/g, ""),
                    __is_new: __is_new,
                    NRAPLICATIVO: 1,
                }
            }).then(() => {
                delete window.shouldUpdateFCMToken;
            });
        } else {
            return Promise.resolve();
        }
    };

    function removeDevice() {
        if ('device' in window) { // if isn't running in device do nothing
            let deviceDataToDelete = {
                UUID: device.uuid,
                NRAPLICATIVO: 1
                // CDOPERADOR: localStorage.getItem('CDOPERADOR').replace(/"/g, ""),
            };
            restEngine.post('/crud/device/remove', {
                "requestType": "Row",
                "row": deviceDataToDelete
            });
        }
    };
};

Configuration(function (ContextRegister) {
    ContextRegister.register('engine', engine);
});