let FCMBootstrap = function( ScreenService, General ) {

    /**
     * @param {string} topic 
     */
    let unsubscribe = ( topic ) => {
        return new Promise((resolve, reject) => {
            if('FirebasePlugin' in window){
                FirebasePlugin.unsubscribe(topic, (s) => {
                    localStorage.removeItem("subscribedTopic");
                    resolve(s);
                }, (e) => reject(e));
            } else {
                reject();
            }
        });
    };

    /**
     * @param {string} topic 
     */
    let subscribe = ( topic ) => {
        return new Promise((resolve, reject) => {
            if('FirebasePlugin' in window){
                FirebasePlugin.subscribe(topic, (s) =>{ 
                    localStorage.setItem("subscribedTopic", topic);
                    resolve(s);
                }, (e) => {
                    // ScreenService.errorNotification("Falha ao se inscrever no gerenciador de notificação");
                    reject(e);
                });
            } else {
                // ScreenService.errorNotification('Falha ao identificar gerenciador de notificação');
                reject();
            }
        });
    };
    
    this.subscribeTopic = ( qrCodeData, customerID ) => {
        let subscribedTopic = localStorage.getItem("subscribedTopic");
        let newTopicName = customerID + "_" + qrCodeData.nrorg + "_" + qrCodeData.nrqrcode;
        if(subscribedTopic){
            return unsubscribe(subscribedTopic).then(() => subscribe( newTopicName ));
        } else {
            return subscribe(newTopicName);
        }
    };

    this.getCustomerIdFromServer = (qrCodeData) => {
        return General.callRestEngine("/getCustomerID", "Row", {}).then( function(response) {
            const { dataset = {} } = response || {}; 
            const { data = {} } = dataset;
            const { customerID = null } = data;
            if ( customerID ) {
                setLocalVar('customerID', customerID);
                return this.subscribeTopic(qrCodeData, customerID);
            } else {
                // ScreenService.errorNotification("Falha ao registrar identificador de notificação");
                return Promise.reject('UNDEFINED_CUSTOMER_ID');
            }
        }.bind(this)).catch(error => {
            // ScreenService.errorNotification("Falha ao recuperar identificador de notificação do servidor");
            return Promise.reject(error);
        });
    };

    this.start = () => {
        let qrCodeData = getLocalVar('qrCodeData');
        let customerID = getLocalVar('customerID');
        if(qrCodeData && customerID){
            return this.subscribeTopic(qrCodeData, customerID);
        } else if(qrCodeData){
            // ScreenService.errorNotification("Falha ao registrar no gerenciador de notificação");
            return this.getCustomerIdFromServer(qrCodeData);
        }
        return Promise.reject("UNEXPECTED_ERROR");
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('FCMBootstrap', FCMBootstrap);
});
